/* This file is part of Eclat.
   Copyright (C) 2012 Sergey Poznyakoff.
 
   Eclat is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.
 
   Eclat is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with Eclat.  If not, see <http://www.gnu.org/licenses/>. */

#include <stdio.h>
#include <ctype.h>
#include <assert.h>

int
comment(FILE *fp, int c)
{
	if (c != '/') {
		ungetc(c, fp);
		return 0;
	}
	
	if ((c = getc(fp)) == '/') {
		while ((c = getc(fp)) != EOF && c != '\n')
			;
		return 1;
	}
	
	if (c == '*') {
		do {
			while ((c = getc(fp)) != '*') {
				if (c == EOF)
					return 0;
			}
		} while ((c = getc(fp)) != '/');
		return 1;
	}
	ungetc(c, fp);
	ungetc('/', fp);
	return 0;
}

int
main(int argc, char **argv)
{
	int i;
	FILE *fp;
	int c;
	char buf[128];
	size_t lev = 0;
	
	for (i = 1; i < argc; i++) {
		fp = fopen(*++argv, "r");
		if (!fp) {
			perror(*argv);
			return 1;
		}

		if (i > 1) {
			/* Skip whitespace and comment lines */
			do {
				while ((c = fgetc(fp)) != EOF && isspace(c))
					;
			} while (c != EOF && comment(fp, c));
			printf(" else ");
		}
		
		while ((c = fgetc(fp)) != EOF) {
			putchar(c);
			if (c == '}') {
				lev = 0;
				while ((c = fgetc(fp)) != EOF &&
				       isspace(c)) {
					assert(lev < sizeof(buf));
					buf[lev++] = c;
				}
				if (c == EOF)
					break;
				else {
					buf[lev++] = c;
					fwrite(buf, lev, 1, stdout);
				}
			}
		}
		fclose(fp);
	}
	return 0;
}
