/* This file is part of Eclat.
   Copyright (C) 2012 Sergey Poznyakoff.
 
   Eclat is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.
 
   Eclat is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with Eclat.  If not, see <http://www.gnu.org/licenses/>. */

#include "eclat.h"

static char *attrs[] = {
	"instanceType",
	"kernel",
	"ramdisk",
	"userData",
	"disableApiTermination",
	"instanceInitiatedShutdownBehavior",
	"rootDeviceName",
	"blockDeviceMapping",
	"sourceDestCheck",
	"groupSet",
	"productCodes",
	"ebsOptimized",
	NULL
};

static void
list_attrs(FILE *fp)
{
	size_t ncols = get_scr_cols();
	int i, col, len;
	char *delim = "";
	
	fprintf(fp, "Available attributes are:\n");
	col = 0;
	for (i = 0; attrs[i]; i++) {
		len = strlen(delim) + strlen(attrs[i]);
		if (col + len > ncols) {
			fprintf(fp, ",\n%s", attrs[i]);
			col = strlen(attrs[i]);
		} else
			col += fprintf(fp, "%s%s", delim, attrs[i]);
		delim = ", ";
	}
	fputc('\n', fp);
	fputc('\n', fp);
}	
	
int
eclat_describe_instance_attribute(CURL *curl, int argc, char **argv)
{
	int i;
	struct ec2_query *q;

	generic_proginfo->args_doc = "INST-ID ATTR";
	generic_proginfo->print_help_hook = list_attrs;
	generic_parse_options("eclat describe-instance-attribute",
			     "describe the specified attribute of the instance",
			      argc, argv, &i);
	argv += i;
	argc -= i;

	if (argc != 2)
		die(EX_USAGE, "wrong number of arguments");

	translate_ids(1, argv, "InstanceId");

	q = eclat_query_create(use_ssl ? EC2_QF_HTTPS : 0, endpoint, "/");
	eclat_query_add_param(q, "Action", "DescribeInstanceAttribute");
	eclat_query_add_param(q, "InstanceId", argv[0]);
	eclat_query_add_param(q, "Attribute", argv[1]);
	
	return eclat_send_query(curl, q);
}
